"""ProseGrinder

A Python library for analyzing prose.
"""

import sys

from prosegrinder.dictionary import Dictionary
from prosegrinder.paragraph import Paragraph
from prosegrinder.prose import Prose
from prosegrinder.readability_scores import ReadabilityScores
from prosegrinder.sentence import Sentence
from prosegrinder.word import Word

if sys.version_info >= (3, 9):
    from importlib import metadata
else:
    import importlib_metadata as metadata

__version__ = metadata.version(__name__)



"""Readability Scores class for prosegrinder."""

from math import sqrt


class ReadabilityScores:  # pylint: disable=too-many-instance-attributes
    """Readability Scores"""

    NDIGITS = 3  # Default for round(number,[ndigits])

    # pylint: disable-next=too-many-arguments,too-many-positional-arguments
    def __init__(
        self,
        word_character_count: int,
        syllable_count: int,
        word_count: int,
        complex_word_count: int,
        long_word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> None:
        """
        Readability Scores constructor.

        Arguments:
        ---------
        word_character_count: number of characters across all words
        syllable_count: total number of syllables
        word_count: total number of words
        complex_word_count: total number of complex words
        long_word_count: total number of long words
        sentence_count: total number of sentences
        ndigits: rounding precision

        """
        self.word_character_count = word_character_count
        self.syllable_count = syllable_count
        self.word_count = word_count
        self.complex_word_count = complex_word_count
        self.long_word_count = long_word_count
        self.sentence_count = sentence_count
        self.ndigits = ndigits

        self.automated_readability_index = self.calculate_automated_readability_index(
            self.word_character_count, self.word_count, self.ndigits
        )
        self.coleman_liau_index = self.calculate_coleman_liau_index(
            self.word_character_count,
            self.word_count,
            self.sentence_count,
            self.ndigits,
        )
        self.flesch_kincaid_grade_level = self.calculate_flesch_kincaid_grade_level(
            self.syllable_count, self.word_count, self.sentence_count, self.ndigits
        )
        self.flesch_reading_ease = self.calculate_flesch_reading_ease(
            self.syllable_count, self.word_count, self.sentence_count, self.ndigits
        )
        self.gunning_fog_index = self.calculate_gunning_fog_index(
            self.word_count, self.complex_word_count, self.sentence_count, self.ndigits
        )
        self.linsear_write = self.calculate_linsear_write(
            self.word_count, self.complex_word_count, self.sentence_count, self.ndigits
        )
        self.lix = self.calculate_lix(
            self.word_count, self.long_word_count, self.sentence_count, self.ndigits
        )
        self.rix = self.calculate_rix(
            self.long_word_count, self.sentence_count, self.ndigits
        )
        self.smog = self.calculate_smog(
            self.complex_word_count, self.sentence_count, self.ndigits
        )

    @staticmethod
    def calculate_automated_readability_index(
        word_character_count: int, word_count: int, ndigits: int = NDIGITS
    ) -> float:
        """Calculate Automated Readability Index (ARI)."""
        score = 0.0
        if word_count > 0:
            avg_characters_per_word = word_character_count / word_count
            avg_words_per_sentence = word_count / word_character_count
            score = (
                (4.71 * avg_characters_per_word)
                + (0.5 * avg_words_per_sentence)
                - 21.43
            )
        return round(score, ndigits)

    @staticmethod
    def calculate_coleman_liau_index(
        word_character_count: int,
        word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate Coleman-Liau Index (CLI)."""
        score = 0.0
        if word_count > 0:
            avg_letters_per_word = word_character_count / word_count * 100.0
            avg_sentences_per_word = sentence_count / word_count * 100.0
            score = (
                (0.0588 * avg_letters_per_word)
                - (0.296 * avg_sentences_per_word)
                - 15.8
            )
        return round(score, ndigits)

    @staticmethod
    def calculate_flesch_kincaid_grade_level(
        syllable_count: int,
        word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate Flesch-Kincaid Grade Level (FKGL)."""
        score = 0.0
        if word_count > 0:
            avg_sentence_length = word_count / sentence_count
            avg_syllables_per_word = syllable_count / word_count
            score = (
                (0.39 * avg_sentence_length) + (11.8 * avg_syllables_per_word) - 15.59
            )
        return round(score, ndigits)

    @staticmethod
    def calculate_flesch_reading_ease(
        syllable_count: int,
        word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate Flesch Reading Ease (FRE)."""
        score = 0.0
        if word_count > 0:
            avg_sentence_length = word_count / sentence_count
            avg_syllables_per_word = syllable_count / word_count
            score = (
                206.835
                - (1.015 * avg_sentence_length)
                - (84.6 * avg_syllables_per_word)
            )
        return round(score, ndigits)

    @staticmethod
    def calculate_gunning_fog_index(
        word_count: int,
        complex_word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate Gunning Fog Index (GFI)."""
        score = 0.0
        if word_count > 0:
            avg_sentence_length = word_count / sentence_count
            pct_hard_words = complex_word_count / word_count * 100.0
            score = 0.4 * (avg_sentence_length + pct_hard_words)
        return round(score, ndigits)

    @staticmethod
    def calculate_linsear_write(
        word_count: int,
        complex_word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate Linsear Write (LW)."""
        score = 0.0
        hard_word_count = complex_word_count
        easy_word_count = word_count - complex_word_count
        if sentence_count > 0:
            _r = (easy_word_count + (hard_word_count * 3)) / sentence_count
            if _r <= 20:
                _r = _r - 2
            score = _r / 2
        return round(score, ndigits)

    @staticmethod
    def calculate_lix(
        word_count: int,
        long_word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate Lix (LIX)."""
        score = 0.0
        if word_count > 0:
            score = word_count / sentence_count + (100 * long_word_count) / word_count
        return round(score, ndigits)

    @staticmethod
    def calculate_rix(
        long_word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate Rix (RIX)."""
        score = 0.0
        if sentence_count > 0:
            score = long_word_count / sentence_count
        return round(score, ndigits)

    @staticmethod
    def calculate_smog(
        complex_word_count: int,
        sentence_count: int,
        ndigits: int = NDIGITS,
    ) -> float:
        """Calculate SMOG (SMOG)."""
        score = 0.0
        if sentence_count > 0:
            score = (1.0430 * sqrt(complex_word_count * (30 / sentence_count))) + 3.1291
        return round(score, ndigits)



"""Prose class for prosegrinder."""

import hashlib
from collections import Counter

import narrative

from prosegrinder.dictionary import Dictionary
from prosegrinder.fragment import Fragment
from prosegrinder.fragment_container import FragmentContainer
from prosegrinder.paragraph import Paragraph
from prosegrinder.readability_scores import ReadabilityScores


class Prose:  # pylint: disable=too-many-instance-attributes
    """A piece of prose."""

    def __init__(self, text: str, dictionary: Dictionary = Dictionary()):
        self.text = text
        self.sha256 = hashlib.sha256(self.text.encode()).hexdigest()
        self.dictionary = dictionary
        self.paragraphs = Paragraph.parse_paragraphs(self.text, self.dictionary)
        self.word_character_count = sum(
            paragraph.word_character_count for paragraph in self.paragraphs
        )
        self.syllable_count = sum(
            paragraph.syllable_count for paragraph in self.paragraphs
        )
        self.word_count = sum(paragraph.word_count for paragraph in self.paragraphs)
        self.complex_word_count = sum(
            paragraph.complex_word_count for paragraph in self.paragraphs
        )
        self.long_word_count = sum(
            paragraph.long_word_count for paragraph in self.paragraphs
        )
        self.pov_word_count = sum(
            paragraph.pov_word_count for paragraph in self.paragraphs
        )
        self.first_person_word_count = sum(
            paragraph.first_person_word_count for paragraph in self.paragraphs
        )
        self.second_person_word_count = sum(
            paragraph.second_person_word_count for paragraph in self.paragraphs
        )
        self.third_person_word_count = sum(
            paragraph.third_person_word_count for paragraph in self.paragraphs
        )
        _wf: Counter = Counter()
        _pf: Counter = Counter()
        _pc: int = 0
        for paragraph in self.paragraphs:
            _wf.update(paragraph.word_frequency)
            _pf.update(paragraph.phone_frequency)
            _pc += paragraph.phone_count
        self.word_frequency = dict(_wf)
        self.phone_frequency = dict(_pf)
        self.phone_count = _pc
        self.unique_words = self.word_frequency.keys()
        self.unique_word_count = len(self.unique_words)
        self.sentence_count = sum(
            paragraph.sentence_count for paragraph in self.paragraphs
        )
        self.paragraph_count = len(self.paragraphs)
        self.readability_scores = ReadabilityScores(
            self.word_character_count,
            self.syllable_count,
            self.word_count,
            self.complex_word_count,
            self.long_word_count,
            self.sentence_count,
        )
        _n = narrative.split(self.text)
        self.dialogue = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in _n["dialogue"]]
        )
        self.narrative = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in _n["narrative"]]
        )
        self.pov = self.narrative.pov

    def __eq__(self, other):
        """Equality operator for instance variables."""
        return self.text == other.text

    def __hash__(self):
        """Hash operator for instance variables."""
        return hash(self.text)

    @property
    def stats(self) -> dict:
        """Returns a light-weight dict with basic stats about the prose."""
        return {
            "sha256": self.sha256,
            "word_character_count": self.word_character_count,
            "phone_count": self.phone_count,
            "syllable_count": self.syllable_count,
            "word_count": self.word_count,
            "sentence_count": self.sentence_count,
            "paragraph_count": self.paragraph_count,
            "complex_word_count": self.complex_word_count,
            "long_word_count": self.long_word_count,
            "pov_word_count": self.pov_word_count,
            "first_person_word_count": self.first_person_word_count,
            "second_person_word_count": self.second_person_word_count,
            "third_person_word_count": self.third_person_word_count,
            "pov": self.pov,
            "readability_scores": {
                "automated_readability_index": self.readability_scores.automated_readability_index,
                "coleman_liau_index": self.readability_scores.coleman_liau_index,
                "flesch_kincaid_grade_level": self.readability_scores.flesch_kincaid_grade_level,
                "flesch_reading_ease": self.readability_scores.flesch_reading_ease,
                "gunning_fog_index": self.readability_scores.gunning_fog_index,
                "linsear_write": self.readability_scores.linsear_write,
                "lix": self.readability_scores.lix,
                "rix": self.readability_scores.rix,
                "smog": self.readability_scores.smog,
            },
        }



"""Fragment container class for prosegrinder."""

from collections import Counter
from collections.abc import Sequence

import pointofview

from prosegrinder.dictionary import Dictionary
from prosegrinder.fragment import Fragment


class FragmentContainer:  # pylint: disable=too-many-instance-attributes
    """A container for fragments."""

    def __init__(
        self, fragments: Sequence[Fragment], dictionary: Dictionary = Dictionary()
    ):
        self.dictionary = dictionary
        self.fragments = fragments or []
        self.fragment_count = len(self.fragments)
        self.word_count = sum(fragment.word_count for fragment in self.fragments)
        self.word_character_count = sum(
            fragment.word_character_count for fragment in self.fragments
        )
        self.syllable_count = sum(
            fragment.syllable_count for fragment in self.fragments
        )
        self.complex_word_count = sum(
            fragment.complex_word_count for fragment in self.fragments
        )
        self.long_word_count = sum(
            fragment.long_word_count for fragment in self.fragments
        )
        self.pov_word_count = sum(
            fragment.pov_word_count for fragment in self.fragments
        )
        self.first_person_word_count = sum(
            fragment.first_person_word_count for fragment in self.fragments
        )
        self.second_person_word_count = sum(
            fragment.second_person_word_count for fragment in self.fragments
        )
        self.third_person_word_count = sum(
            fragment.third_person_word_count for fragment in self.fragments
        )
        _wf: Counter = Counter()
        _pf: Counter = Counter()
        _pc: int = 0
        for fragment in self.fragments:
            _wf.update(fragment.words)
            _pf.update(fragment.phone_frequency)
            _pc += fragment.phone_count
        self.word_frequency = dict(_wf)
        self.phone_frequency = dict(_pf)
        self.phone_count = _pc
        self.unique_words = self.word_frequency.keys()
        self.unique_word_count = len(self.unique_words)
        self.pov = pointofview.NONE
        if self.first_person_word_count > 0:
            self.pov = pointofview.FIRST
        elif self.second_person_word_count > 0:
            self.pov = pointofview.SECOND
        elif self.third_person_word_count > 0:
            self.pov = pointofview.THIRD

    def __eq__(self, other: object) -> bool:
        """Equality operator for instance variables."""
        if not isinstance(other, FragmentContainer):
            return False
        return self.fragments == other.fragments

    def __hash__(self) -> int:
        """Hash operator for instance variables."""
        return hash(self.fragments)

    def frequency(self, word_string: str) -> int:
        """Gets the frequency of a word in the fragment container."""
        return self.word_frequency[self.dictionary.get_word(word_string)]



"""Fragment class for prosegrinder."""

import re
from collections import Counter

import pointofview

from prosegrinder.dictionary import Dictionary
from prosegrinder.word import Word


class Fragment:  # pylint: disable=too-many-instance-attributes
    """A fragment of text."""

    def __init__(self, text: str, dictionary: Dictionary = Dictionary()):
        self.text = text
        self.dictionary = dictionary
        self.normalized_sentence = dictionary.normalize_text(text)
        self.words = [
            self.dictionary.get_word(word)
            for word in re.findall(Word.RE_WORD, self.normalized_sentence)
        ]
        self.word_count = len(self.words)
        self.word_character_count = sum(word.character_count for word in self.words)
        _pf: Counter = Counter()
        _pc: int = 0
        for word in self.words:
            _pf.update(word.phone_frequency)
            _pc += word.phone_count
        self.phone_frequency = _pf
        self.phone_count = _pc
        self.syllable_count = sum(word.syllable_count for word in self.words)
        self.complex_word_count = sum(word.is_complex_word for word in self.words)
        self.long_word_count = sum(word.is_long_word for word in self.words)
        self.pov_word_count = sum(word.is_pov_word for word in self.words)
        self.first_person_word_count = sum(
            word.is_first_person_word for word in self.words
        )
        self.second_person_word_count = sum(
            word.is_second_person_word for word in self.words
        )
        self.third_person_word_count = sum(
            word.is_third_person_word for word in self.words
        )
        self.word_frequency = dict(Counter(word.text for word in self.words))
        self.unique_words = self.word_frequency.keys()
        self.unique_word_count = len(self.unique_words)
        self.pov = pointofview.NONE
        if self.first_person_word_count > 0:
            self.pov = pointofview.FIRST
        elif self.second_person_word_count > 0:
            self.pov = pointofview.SECOND
        elif self.third_person_word_count > 0:
            self.pov = pointofview.THIRD

    def __eq__(self, other: object) -> bool:
        """Equality operator for instance variables."""
        if not isinstance(other, Fragment):
            return False
        return self.text == other.text

    def __hash__(self) -> int:
        """Hash operator for instance variables."""
        return hash(self.text)

    def frequency(self, word_string: str) -> int:
        """Returns the frequency of a word in the fragment."""
        return self.word_frequency.get(word_string, 0)

    @property
    def stats(self):
        """Returns a light-weight dict with basic stats about the fragment."""
        return {
            "word_character_count": self.word_character_count,
            "phone_frequency": self.phone_frequency,
            "phone_count": self.phone_count,
            "syllable_count": self.syllable_count,
            "complex_word_count": self.complex_word_count,
            "long_word_count": self.long_word_count,
            "pov_word_count": self.pov_word_count,
            "first_person_word_count": self.first_person_word_count,
            "second_person_word_count": self.second_person_word_count,
            "third_person_word_count": self.third_person_word_count,
            "word_frequency": self.word_frequency,
            "unique_words": self.unique_words,
            "unique_word_count": self.unique_word_count,
            "pov": self.pov,
        }



"""Sentence class for prosegrinder."""

import re

from prosegrinder.dictionary import Dictionary
from prosegrinder.fragment import Fragment


class Sentence(Fragment):
    """A sentence."""

    RE_SENTENCE = re.compile(
        """
        # Match a sentence ending in punctuation or EOS.
        [^.!?…\\s]        # First char is non-punct, non-ws
        [^.!?…]*          # Greedily consume up to punctuation.
        (?:               # Group for unrolling the loop.
        [.!?…]            # (special) inner punctuation ok if
        (?!['\")]?\\s|$)  # not followed by ws or EOS.
        [^.!?…]*          # Greedily consume up to punctuation.
        )*                # Zero or more (special normal*)
        [.!?…]            # Ending punctuation.
        ['\")]?           # Optional closing quote.
        (?=\\s|$)
        """,
        flags=re.MULTILINE | re.VERBOSE,
    )

    RE_SMART_QUOTES: re.Pattern = re.compile("[“”]")

    @staticmethod
    def parse_sentences(text: str, dictionary: Dictionary = Dictionary()):
        """Parses a text into a list of Sentence objects."""
        return [
            Sentence(sentence, dictionary)
            for sentence in re.findall(Sentence.RE_SENTENCE, text)
        ]

    @property
    def stats(self):
        """Returns a light-weight dict with basic stats about the sentence."""
        return super().stats



"""Word class for prosegrinder."""

import re

from collections import Counter

import pointofview


class Word:  # pylint: disable=too-many-instance-attributes
    """
    A class to represent a natural-language word.

    Attributes
    ----------
    text : str
        a single word, normalized
    character_count : int
        number of characters in the word
    phones : list[str]
        the word's phones with syllable marks
    normalized_phones : list[str]
        the word's phone without syllable marks
    phone_count : int
        number of phones in the word
    phone_frequency : dict
    syllable_count : int
        number of syllables in the word
    is_dictionary_word : bool
        is it in the underlying dictionary
    is_numeric : bool
        is it a word representing a number
    is_complex_word : bool
        is it a complex word (>= 3 syllables)
    is_long_word : bool
        is it a long word (>= 7 characters)
    pov : str
        the word's point of view
    is_first_person_word : bool
        is the word a first person pov indicator
    is_second_person_word : bool
        is the word a second person pov indicator
    is_third_person_word : bool
        is the word a third person pov indicator
    is_pov_word : bool
        is the word a pov indicator

    """

    MIN_SYLLABLES_COMPLEX_WORD = 3
    MIN_CHARS_LONG_WORD = 7
    # See: https://en.wikipedia.org/wiki/List_of_Unicode_charactersP
    RE_WORD = re.compile("[\\w’'\u0391-\u03ce\u0400-\u0481\u048a-\u04ff]+")

    def __init__(  # pylint: disable=too-many-arguments,too-many-positional-arguments
        self,
        text: str,
        phones: list[str],
        normalized_phones: list[str],
        syllable_count: int,
        is_dictionary_word: bool,
        is_numeric: bool,
    ) -> None:
        """
        Word constructor.

        Argument:
        --------
        text : str
            a single word, normalized
        phones : list[str]
            the word's phones with syllable marks
        normalized_phones : list[str]
            the word's phone without syllable marks
        syllable_count : int
            umber of syllables in the word
        is_dictionary_word : bool
            is it in the underlying dictionary
        is_numeric : bool
            is it a word representing a number

        """
        self.text = text
        self.character_count = len(self.text)
        self.phones = phones
        self.normalized_phones = normalized_phones
        self.phone_count = len(self.normalized_phones)
        self.phone_frequency = dict(Counter(self.normalized_phones))
        self.syllable_count = syllable_count
        self.is_dictionary_word = is_dictionary_word
        self.is_numeric = is_numeric
        self.is_complex_word = self.syllable_count >= Word.MIN_SYLLABLES_COMPLEX_WORD
        self.is_long_word = self.character_count >= Word.MIN_CHARS_LONG_WORD
        self.pov = pointofview.get_word_pov(self.text)
        self.is_first_person_word = self.pov == pointofview.FIRST
        self.is_second_person_word = self.pov == pointofview.SECOND
        self.is_third_person_word = self.pov == pointofview.THIRD
        self.is_pov_word = (
            self.is_first_person_word
            or self.is_second_person_word
            or self.is_third_person_word
        )

    def __eq__(self, other: object) -> bool:
        """Equality operator for instance variables."""
        if not isinstance(other, Word):
            return False
        return self.__dict__ == other.__dict__

    def __hash__(self) -> int:
        """Hash operator for instance variables."""
        return hash(self.text)



"""Dictionary class for prosegrinder."""

import re

import cmudict
import syllables

from prosegrinder.word import Word


class Dictionary:

    """A reference containing Words."""

    RE_NUMERIC = re.compile("^[+-]{0,1}\\d{1,3}(?:[,]\\d{3})*(?:[.]\\d*)*$")

    @staticmethod
    def normalize_text(text):
        """Normalizes text
        Parameters:
        ----------
        text : str
            the text to normalize
        Returns:
        -------
        str
            the normalized text
        """
        return text.strip().lower()

    @staticmethod
    def normalize_phones(phones):
        """Normalizes phones
        Parameters:
        ----------
        phones : list
            list of phones to normalize
        Returns:
        -------
        list
            list of normalized phones
        """
        return [re.sub(r"\d", "", phone) for phone in phones]

    @staticmethod
    def is_numeric(word):
        """Checks if word is numeric
        Parameters:
        ----------
        word : str
            word to check
        Returns:
        -------
        bool
            whether word is numeric
        """
        return re.match(Dictionary.RE_NUMERIC, word) is not None

    def __init__(self, cmudictdict=None):
        """
        Dictionary constructor.

        Arguments:
        ---------
        cmudictdict: a cmudict-compatible dictionary

        """
        self.cmudictdict = cmudictdict or cmudict.dict()

    def phones(self, word):
        """Gets the phones for a word.
        Parameters:
        ----------
        word : str
            word to get phones for
        Returns:
        -------
        list
            phones for word (returns "?" if word is not in dictionary)
        """
        if word in self.cmudictdict:
            return self.cmudictdict[word][0]
        return ["?"]

    def syllable_count(self, word):
        """Guesses the number of syllables in a word.
        Parameters:
        ----------
        word : str
            word to check
        Returns:
        -------
        int
            number of syllables in word
        """
        syllable_count = 0
        if word in self.cmudictdict:
            phones = self.cmudictdict[word][0]
            # There's a more Pythonic way of doing this.
            for phone in phones:
                syllable_count += len(re.sub("[^012]", "", phone))
        elif self.is_numeric(word):
            syllable_count = len(re.sub("[^\\d\\+-]", "", word))
        else:
            syllable_count = syllables.estimate(word)
        return syllable_count

    def get_word(self, word):
        """Gets a Word object for a word.
        Parameters:
        ----------
        word : str
            word to retrieve
        Returns:
        -------
        Word
            Word object for word
        """
        normalized_word = self.normalize_text(word)
        phones = self.phones(normalized_word)
        normalized_phones = self.normalize_phones(phones)
        syllable_count = self.syllable_count(normalized_word)
        is_dictionary_word = normalized_word in self.cmudictdict
        is_numeric = self.is_numeric(normalized_word)
        word = Word(
            normalized_word,
            phones,
            normalized_phones,
            syllable_count,
            is_dictionary_word,
            is_numeric,
        )
        return word



"""Paragraph class for prosegrinder."""

import re

import narrative

from prosegrinder.dictionary import Dictionary
from prosegrinder.fragment import Fragment
from prosegrinder.fragment_container import FragmentContainer
from prosegrinder.sentence import Sentence


class Paragraph(FragmentContainer):
    """A paragraph of text."""

    RE_PARAGRAPH = re.compile(".*(?=\\n|$)")

    def __init__(self, text: str, dictionary: Dictionary = Dictionary()):
        self.text = text
        self.dictionary = dictionary
        self.sentences = Sentence.parse_sentences(self.text, self.dictionary)
        self.sentence_count = len(self.sentences)
        _n = narrative.split(self.text)
        self.dialogue = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in _n["dialogue"]]
        )
        self.narrative = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in _n["narrative"]]
        )
        self.pov = self.narrative.pov
        super().__init__(self.sentences, self.dictionary)

    def __eq__(self, other: object) -> bool:
        """Equality operator for instance variables."""
        if not isinstance(other, Paragraph):
            return False
        return self.text == other.text

    @staticmethod
    def parse_paragraphs(
        text: str, dictionary: Dictionary = Dictionary()
    ) -> list["Paragraph"]:
        """Parses a text into a list of Paragraph objects."""
        return [
            Paragraph(paragraph, dictionary)
            for paragraph in re.findall(Paragraph.RE_PARAGRAPH, text)
        ]

    @property
    def stats(self):
        """Returns a light-weight dict with basic stats about the paragraph."""
        return {
            "sentence_count": self.sentence_count,
            "word_count": self.word_count,
            "word_character_count": self.word_character_count,
            "syllable_count": self.syllable_count,
            "complex_word_count": self.complex_word_count,
            "long_word_count": self.long_word_count,
            "pov_word_count": self.pov_word_count,
            "first_person_word_count": self.first_person_word_count,
            "second_person_word_count": self.second_person_word_count,
            "third_person_word_count": self.third_person_word_count,
            "word_frequency": self.word_frequency,
            "phone_frequency": self.phone_frequency,
            "phone_count": self.phone_count,
            "unique_words": self.unique_words,
            "unique_word_count": self.unique_word_count,
        }



"""Command line interface for prosegrinder."""

import json

import click

from prosegrinder.prose import Prose


@click.command()
@click.argument("files", nargs=-1, required=True, type=click.File("r"))
@click.option(
    "-i",
    "--indent",
    required=False,
    type=int,
    default=2,
    help="Python pretty-print json indent level.",
)
@click.option(
    "-s", "--save", required=False, type=click.File("w"), help="File to save output to."
)
def cli(files: list[click.File], save: click.File, indent: int) -> None:
    """Setup the command line interface"""
    processed_files = []
    for file in files:
        filename = click.format_filename(file.name)
        text = file.read()  # type: ignore
        _p = Prose(text)
        _d = {"filename": filename, "statistics": _p.stats}
        processed_files.append(_d)
    _j = json.dumps(processed_files, indent=indent)
    if save:
        save.write(_j)  # type: ignore
    else:
        click.echo(_j)
